"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sign_1 = require("./lib/sign");
const api_1 = require("./api");
const config_1 = require("./config");
const util_1 = require("./utils/util");
const check_1 = require("./utils/check");
const sha256_1 = __importDefault(require("crypto-js/sha256"));
const types_1 = require("./types");
const errors_1 = require("./utils/errors");
const ethers_1 = require("ethers");
const uuid_1 = require("uuid");
const popup_1 = require("./lib/popup");
const constants_1 = require("./constants");
const hashPersonalMessage_1 = __importDefault(require("./lib/hashPersonalMessage"));
const isString_1 = __importDefault(require("lodash/isString"));
__exportStar(require("./types"), exports);
const _cachedInfo = {};
const cacheHelper = async (key, host) => {
    var _a, _b, _c;
    const timestamp = (0, util_1.getTimestamp)();
    // cache info 3 mins
    if (((_a = _cachedInfo[key]) === null || _a === void 0 ? void 0 : _a.value) != null &&
        _cachedInfo[key].timestamp > timestamp - 3 * 60) {
        return (_b = _cachedInfo[key]) === null || _b === void 0 ? void 0 : _b.value;
    }
    if (key === 'everpay') {
        const value = await (0, api_1.getEverpayInfo)(host);
        _cachedInfo[key] = { value, timestamp };
    }
    else if (key === 'express') {
        const value = await (0, api_1.getExpressInfo)(host);
        _cachedInfo[key] = { value, timestamp };
    }
    return (_c = _cachedInfo[key]) === null || _c === void 0 ? void 0 : _c.value;
};
class Everpay extends types_1.EverpayBase {
    constructor(config) {
        var _a, _b;
        super();
        this.getAccountChainType = util_1.getAccountChainType;
        this._config = {
            ...config,
            account: (_a = config === null || config === void 0 ? void 0 : config.account) !== null && _a !== void 0 ? _a : '',
            chainType: (_b = config === null || config === void 0 ? void 0 : config.chainType) !== null && _b !== void 0 ? _b : types_1.ChainType.ethereum
        };
        this._apiHost = (0, config_1.getEverpayHost)(config === null || config === void 0 ? void 0 : config.debug);
        this._expressHost = (0, config_1.getExpressHost)(config === null || config === void 0 ? void 0 : config.debug);
        this._explorerHost = (0, config_1.getExplorerHost)(config === null || config === void 0 ? void 0 : config.debug);
    }
    async info() {
        const result = await cacheHelper('everpay', this._apiHost);
        return result;
    }
    async expressInfo() {
        const result = await cacheHelper('express', this._expressHost);
        return result;
    }
    async getEmailRegisterData() {
        const result = await (0, api_1.getEmailRegisterData)(this._apiHost, this._config.account);
        return result;
    }
    async getAccountData() {
        const acc = (0, util_1.isSmartAccount)(this._config.account) ? (0, util_1.genEverId)(this._config.account) : this._config.account;
        const result = await (0, api_1.getAccountData)(this._apiHost, acc);
        return result;
    }
    async balance(params) {
        var _a;
        await this.info();
        const { tag, account } = params;
        const accTemp = account !== null && account !== void 0 ? account : this._config.account;
        const acc = (0, util_1.isSmartAccount)(accTemp) ? (0, util_1.genEverId)(accTemp) : accTemp;
        const token = (0, util_1.getTokenByTag)(tag, (_a = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _a === void 0 ? void 0 : _a.value.tokenList);
        (0, check_1.checkParams)({ account: acc, tag, token });
        const mergedParams = {
            tokenTag: (0, util_1.genTokenTag)(token),
            account: acc
        };
        const everpayBalance = await (0, api_1.getEverpayBalance)(this._apiHost, mergedParams);
        return (0, util_1.fromDecimalToUnit)(everpayBalance.balance.amount, everpayBalance.balance.decimals);
    }
    async balances(params) {
        const info = await this.info();
        params = (params !== null && params !== void 0 ? params : {});
        const { account } = params;
        const accTemp = account !== null && account !== void 0 ? account : this._config.account;
        const acc = (0, util_1.isSmartAccount)(accTemp) ? (0, util_1.genEverId)(accTemp) : accTemp;
        (0, check_1.checkParams)({ account: acc });
        const mergedParams = {
            account: acc
        };
        const everpayBalances = await (0, api_1.getEverpayBalances)(this._apiHost, mergedParams);
        const argTag = !this._config.debug ? 'bsc-arg-0xb5eadfdbdb40257d1d24a1432faa2503a867c270' : 'bsc-arg-0x7846cf6e181bb5c909d6010d15af5fffd3b61229';
        const fraTag = !this._config.debug ? 'bsc-fra-0xeb042ffdabc535de2716c6b51a965f124050d4e1' : 'bsc-fra-0xa98242557818f0135b2381893caec3d4a64f88e5';
        const deleteTag = [
            argTag,
            fraTag
        ];
        const balances = everpayBalances.balances.filter((t) => t.tag !== deleteTag[0] && t.tag !== deleteTag[1]).map(item => {
            const tag = item.tag;
            const token = info.tokenList.find(token => token.tag === tag);
            return {
                chainType: token === null || token === void 0 ? void 0 : token.chainType,
                symbol: token === null || token === void 0 ? void 0 : token.symbol.toUpperCase(),
                tag: token === null || token === void 0 ? void 0 : token.tag,
                address: token === null || token === void 0 ? void 0 : token.id,
                balance: (0, util_1.fromDecimalToUnit)(item.amount, item.decimals)
            };
        });
        return balances;
    }
    async getMergedTxsParams(params) {
        var _a;
        const { page, tag, action, withoutAction, cursor } = params;
        const mergedParams = {};
        if (page !== undefined) {
            mergedParams.page = page;
        }
        if (cursor !== undefined) {
            mergedParams.cursor = cursor;
        }
        if (tag !== undefined) {
            await this.info();
            const token = (0, util_1.getTokenByTag)(tag, (_a = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _a === void 0 ? void 0 : _a.value.tokenList);
            (0, check_1.checkParams)({ token });
            mergedParams.tokenTag = token.tag;
        }
        if (action !== undefined) {
            (0, check_1.checkParams)({ action });
            mergedParams.action = action;
        }
        if (withoutAction !== undefined) {
            mergedParams.withoutAction = withoutAction;
        }
        return mergedParams;
    }
    async txs(params) {
        const mergedParams = await this.getMergedTxsParams(params);
        return await (0, api_1.getExplorerTransactions)(this._explorerHost, mergedParams);
    }
    async txsByAccount(params) {
        var _a;
        const accTemp = (_a = params.account) !== null && _a !== void 0 ? _a : this._config.account;
        const acc = (0, util_1.isSmartAccount)(accTemp) ? (0, util_1.genEverId)(accTemp) : accTemp;
        (0, check_1.checkParams)({ account: acc });
        const mergedParams = await this.getMergedTxsParams(params);
        mergedParams.account = acc;
        return await (0, api_1.getExplorerTransactions)(this._explorerHost, mergedParams);
    }
    async txByHash(everHash) {
        (0, check_1.checkParams)({ everHash });
        return await (0, api_1.getExplorerTransaction)(this._explorerHost, everHash);
    }
    async mintedTxByChainTxHash(chainTxHash) {
        (0, check_1.checkParams)({ chainTxHash });
        return await (0, api_1.getMintdEverpayTransactionByChainTxHash)(this._apiHost, chainTxHash);
    }
    async fees() {
        return await (0, api_1.getFees)(this._apiHost);
    }
    async fee(tag) {
        var _a;
        await this.info();
        const token = (0, util_1.getTokenByTag)(tag, (_a = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _a === void 0 ? void 0 : _a.value.tokenList);
        (0, check_1.checkParams)({ tag, token });
        return await (0, api_1.getFee)(this._apiHost, (0, util_1.genTokenTag)(token));
    }
    async smartAccountAuth(logo, email, emailEditable) {
        const debug = Boolean(this._config.debug);
        email = (0, util_1.isSmartAccount)(email) ? email : '';
        // eslint-disable-next-line no-unneeded-ternary
        emailEditable = emailEditable === false ? false : true;
        const url = `https://app${debug ? '-dev' : ''}.everpay.io/auth?host=${encodeURIComponent(window.location.host)}&logo=${encodeURIComponent(logo)}&version=${constants_1.EVERPAY_JS_VERSION}&email=${email}&editable=${emailEditable ? 1 : 0}`;
        // const url = `http://localhost:8080/auth?host=${encodeURIComponent(window.location.host)}&logo=${encodeURIComponent(logo)}&version=${EVERPAY_JS_VERSION}&email=${email}&editable=${emailEditable ? 1 : 0}`
        const popup = await (0, popup_1.openPopup)(url);
        return await (0, popup_1.runPopup)({
            popup,
            type: 'auth'
        });
    }
    genEverId(account) {
        const acc = account !== null && account !== void 0 ? account : this._config.account;
        return (0, util_1.isSmartAccount)(acc) ? (0, util_1.genEverId)(acc) : acc;
    }
    hashMessage(message) {
        const personalMsgHashBuffer = (0, hashPersonalMessage_1.default)(Buffer.from(message));
        const personalMsgHex = `0x${personalMsgHashBuffer.toString('hex')}`;
        return personalMsgHex;
    }
    async signMessage(message, smartAccountDirectly) {
        if (!(0, isString_1.default)(message)) {
            throw new Error(errors_1.ERRORS.MESSAGE_INCORRECT);
        }
        if ((0, util_1.isSmartAccount)(this._config.account)) {
            if (smartAccountDirectly != null) {
                if (message.length >= 96) {
                    const { sig } = await (0, sign_1.signMessageAsync)(this._config, message, undefined, true);
                    return { message, sig };
                }
                else {
                    throw new Error(errors_1.ERRORS.MESSAGE_INCORRECT);
                }
            }
            else {
                const personalMsgHashBuffer = (0, hashPersonalMessage_1.default)(Buffer.from(message));
                const messageHash = `0x${personalMsgHashBuffer.toString('hex')}`;
                const { sig } = await (0, sign_1.signMessageAsync)(this._config, message);
                return { message: messageHash, sig };
            }
        }
        else {
            const { sig } = await (0, sign_1.signMessageAsync)(this._config, message);
            return { message, sig };
        }
    }
    async verifyMessage(params) {
        const { account, message, type, sig } = params;
        const everId = this.genEverId(account);
        let messageHash = '';
        if ((0, util_1.isEthereumAddress)(account)) {
            const personalMsgHashBuffer = (0, hashPersonalMessage_1.default)(Buffer.from(message));
            messageHash = `0x${personalMsgHashBuffer.toString('hex')}`;
        }
        else if ((0, util_1.isArweaveAddress)(account)) {
            messageHash = `0x${(0, sha256_1.default)(message).toString()}`;
        }
        else if ((0, util_1.isSmartAccount)(account)) {
            messageHash = `0x${(0, util_1.uint8ArrayToHex)(Buffer.from(message))}`;
        }
        else {
            throw new Error(errors_1.ERRORS.ACCOUNT_INVALID);
        }
        const [splitSig, splitPublic] = sig.split(',');
        return await (0, api_1.verifySig)(this._apiHost, {
            account: everId,
            message: messageHash,
            type,
            sig: splitSig,
            public: splitPublic
        });
    }
    async deposit(params) {
        var _a, _b;
        await this.info();
        const { amount, tag } = params;
        const from = this._config.account;
        const token = (0, util_1.getTokenByTag)(tag, (_a = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _a === void 0 ? void 0 : _a.value.tokenList);
        const chainType = this._config.chainType;
        (0, check_1.checkParams)({ account: from, tag, token, amount });
        // arweave 上的 PST 充值必须是整数
        if ((0, util_1.isArweaveChainPSTMode)(token) && chainType === types_1.ChainType.arweave && !(0, util_1.isArweaveL2PSTTokenSymbol)(token.symbol) && parseInt(amount) !== +amount) {
            throw new Error(errors_1.ERRORS.DEPOSIT_ARWEAVE_PST_MUST_BE_INTEGER);
        }
        const chainDecimal = (0, util_1.getChainDecimalByChainType)(token, (0, util_1.isArweaveAOSTestToken)(token) ? types_1.ChainType.aostest : chainType);
        const value = ethers_1.utils.parseUnits((0, util_1.toBN)(amount).toString(), chainDecimal);
        return await (0, sign_1.transferAsync)(this._config, (_b = _cachedInfo.everpay) === null || _b === void 0 ? void 0 : _b.value, {
            symbol: token.symbol,
            token,
            from: from !== null && from !== void 0 ? from : '',
            value
        });
    }
    // amount 为实际收款数量
    async getEverpayTxWithoutSig(type, params) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q;
        if (((_a = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _a === void 0 ? void 0 : _a.value) == null) {
            await this.info();
        }
        const { tag, amount, fee, quickMode } = params;
        const token = (0, util_1.getTokenByTag)(tag, (_b = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _b === void 0 ? void 0 : _b.value.tokenList);
        const from = this._config.account;
        let data = params.data;
        let to = params === null || params === void 0 ? void 0 : params.to;
        let decimalFeeBN = (0, util_1.toBN)(0);
        let decimalOperateAmountBN = (0, util_1.toBN)(0);
        let action = types_1.EverpayAction.transfer;
        (0, check_1.checkParams)({ account: from, tag, token, to });
        if (type === 'transfer') {
            (0, check_1.checkParams)({ amount });
            action = types_1.EverpayAction.transfer;
            decimalOperateAmountBN = (0, util_1.fromUnitToDecimalBN)(amount, (_c = token === null || token === void 0 ? void 0 : token.decimals) !== null && _c !== void 0 ? _c : 0);
        }
        else if (type === 'bundle') {
            action = types_1.EverpayAction.bundle;
            decimalOperateAmountBN = (0, util_1.fromUnitToDecimalBN)(amount, (_d = token === null || token === void 0 ? void 0 : token.decimals) !== null && _d !== void 0 ? _d : 0);
            // eslint-disable-next-line @typescript-eslint/strict-boolean-expressions
        }
        else if (type === 'set') {
            action = types_1.EverpayAction.set;
            decimalOperateAmountBN = (0, util_1.fromUnitToDecimalBN)(amount, (_e = token === null || token === void 0 ? void 0 : token.decimals) !== null && _e !== void 0 ? _e : 0);
        }
        else if (type === 'setAcc') {
            action = types_1.EverpayAction.setAcc;
            decimalOperateAmountBN = (0, util_1.fromUnitToDecimalBN)(amount, (_f = token === null || token === void 0 ? void 0 : token.decimals) !== null && _f !== void 0 ? _f : 0);
        }
        else if (type === 'withdraw') {
            (0, check_1.checkParams)({ amount });
            const chainType = params.chainType;
            // PST 提现到 arweave 网络必须是整数
            if ((0, util_1.isArweaveChainPSTMode)(token) && chainType === types_1.ChainType.arweave && !(0, util_1.isArweaveL2PSTTokenSymbol)(token === null || token === void 0 ? void 0 : token.symbol) && parseInt(amount) !== +amount) {
                throw new Error(errors_1.ERRORS.PST_WITHDARW_TO_ARWEAVE_MUST_BE_INTEGER);
            }
            const balance = await this.balance({ tag });
            const decimalBalanceBN = (0, util_1.fromUnitToDecimalBN)(balance, (_g = token === null || token === void 0 ? void 0 : token.decimals) !== null && _g !== void 0 ? _g : 0);
            // 快速提现
            if (quickMode === true) {
                action = types_1.EverpayAction.transfer;
                const expressInfo = await this.expressInfo();
                const tokenTag = (0, util_1.genTokenTag)(token);
                const foundExpressTokenData = expressInfo.tokens.find(t => (0, util_1.matchTokenTag)(tokenTag, t.tokenTag));
                if (foundExpressTokenData == null) {
                    throw new Error(errors_1.ERRORS.WITHDRAW_TOKEN_NOT_SUPPORT_QUICK_MODE);
                }
                const quickWithdrawLimitBN = (0, util_1.fromUnitToDecimalBN)(foundExpressTokenData.walletBalance, (_h = token === null || token === void 0 ? void 0 : token.decimals) !== null && _h !== void 0 ? _h : 0);
                // 快速提现的手续费，只放入 data 字段中
                const quickWithdrawFeeBN = fee !== undefined
                    ? (0, util_1.fromUnitToDecimalBN)(fee, (_j = token === null || token === void 0 ? void 0 : token.decimals) !== null && _j !== void 0 ? _j : 0)
                    : (0, util_1.toBN)(foundExpressTokenData.withdrawFee);
                // 快速提现的 amount 为全部数量
                decimalOperateAmountBN = (0, util_1.fromUnitToDecimalBN)(amount, (_k = token === null || token === void 0 ? void 0 : token.decimals) !== null && _k !== void 0 ? _k : 0).plus(quickWithdrawFeeBN);
                if (decimalOperateAmountBN.gt(decimalBalanceBN)) {
                    throw new Error(errors_1.ERRORS.WITHDRAW_AMOUNT_LESS_THAN_FEE);
                }
                if (decimalOperateAmountBN.gt(quickWithdrawLimitBN)) {
                    throw new Error(errors_1.ERRORS.INSUFFICIENT_QUICK_WITHDRAWAL_AMOUNT);
                }
                const expressData = (0, util_1.genExpressData)({
                    chainType, to, fee: quickWithdrawFeeBN.toString()
                });
                data = data !== undefined ? { ...data, ...expressData } : { ...expressData };
                // to 需要更改为快速提现收款账户
                to = expressInfo.address;
                // 普通提现
            }
            else {
                action = types_1.EverpayAction.withdraw;
                if (fee !== undefined) {
                    decimalFeeBN = (0, util_1.fromUnitToDecimalBN)(fee, (_l = token === null || token === void 0 ? void 0 : token.decimals) !== null && _l !== void 0 ? _l : 0);
                }
                else {
                    const feeItem = await (0, api_1.getFee)(this._apiHost, (0, util_1.genTokenTag)(token));
                    decimalFeeBN = (0, util_1.toBN)((_m = (0, util_1.getTokenBurnFeeByChainType)(token, feeItem, chainType)) !== null && _m !== void 0 ? _m : '0');
                }
                const targetChainType = chainType;
                data = data !== undefined ? { ...data, targetChainType } : { targetChainType };
                decimalOperateAmountBN = (0, util_1.fromUnitToDecimalBN)(amount, (_o = token === null || token === void 0 ? void 0 : token.decimals) !== null && _o !== void 0 ? _o : 0);
                if (decimalOperateAmountBN.plus(decimalFeeBN).gt(decimalBalanceBN)) {
                    throw new Error(errors_1.ERRORS.WITHDRAW_AMOUNT_LESS_THAN_FEE);
                }
            }
        }
        const everpayTxWithoutSig = {
            tokenSymbol: token === null || token === void 0 ? void 0 : token.symbol,
            action,
            from: (0, util_1.isSmartAccount)(from) ? (0, util_1.genEverId)(from) : from,
            to: (0, util_1.isSmartAccount)(to) ? (0, util_1.genEverId)(to) : to,
            amount: decimalOperateAmountBN.toString(),
            fee: decimalFeeBN.toString(),
            feeRecipient: (_q = (_p = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _p === void 0 ? void 0 : _p.value.feeRecipient) !== null && _q !== void 0 ? _q : '',
            nonce: Date.now().toString(),
            tokenID: token === null || token === void 0 ? void 0 : token.id,
            chainType: token === null || token === void 0 ? void 0 : token.chainType,
            chainID: token === null || token === void 0 ? void 0 : token.chainID,
            data: data !== undefined ? JSON.stringify(data) : '',
            version: config_1.everpayTxVersion
        };
        return everpayTxWithoutSig;
    }
    getEverpayTxMessage(everpayTxWithoutSig) {
        return (0, sign_1.getEverpayTxMessage)(everpayTxWithoutSig);
    }
    async signedEverpayTx(everpayTxWithoutSig) {
        const messageData = (0, sign_1.getEverpayTxMessage)(everpayTxWithoutSig);
        const { sig, everHash } = await (0, sign_1.signMessageAsync)(this._config, messageData);
        const everpayTx = {
            ...everpayTxWithoutSig,
            sig
        };
        return { everpayTx, everHash };
    }
    // sig redpacket
    async signedRedPackTx(redPackTxSig) {
        const messageData = (0, sign_1.getRedPackTxMessage)(redPackTxSig);
        const { sig, everHash } = await (0, sign_1.signMessageAsync)(this._config, messageData);
        const redpackTx = {
            ...redPackTxSig,
            signature: sig
        };
        return { redpackTx, everHash };
    }
    async sendEverpayTx(everpayTxWithoutSig) {
        const { everpayTx, everHash } = await this.signedEverpayTx(everpayTxWithoutSig);
        const postEverpayTxResult = await (0, api_1.postTx)(this._apiHost, everpayTx);
        return {
            ...postEverpayTxResult,
            everpayTx,
            everHash
        };
    }
    async register(params, attachment) {
        var _a, _b, _c, _d;
        if (((_a = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _a === void 0 ? void 0 : _a.value) == null) {
            await this.info();
        }
        const everpayInfo = (_b = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _b === void 0 ? void 0 : _b.value;
        const token = everpayInfo.tokenList.find((t) => {
            return t.symbol.toUpperCase() === 'ETH';
        });
        const everpayTxWithoutSig = {
            tokenSymbol: token === null || token === void 0 ? void 0 : token.symbol,
            action: 'register',
            from: (0, util_1.isSmartAccount)(this._config.account) ? (0, util_1.genEverId)(this._config.account) : this._config.account,
            to: (0, util_1.isSmartAccount)(this._config.account) ? (0, util_1.genEverId)(this._config.account) : this._config.account,
            amount: '0',
            fee: '0',
            feeRecipient: (_d = (_c = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _c === void 0 ? void 0 : _c.value.feeRecipient) !== null && _d !== void 0 ? _d : '',
            nonce: Date.now().toString(),
            tokenID: token === null || token === void 0 ? void 0 : token.id,
            chainType: token === null || token === void 0 ? void 0 : token.chainType,
            chainID: token === null || token === void 0 ? void 0 : token.chainID,
            data: params !== undefined ? JSON.stringify({ mailVerify: params }) : '',
            version: config_1.everpayTxVersion
        };
        const messageData = (0, sign_1.getEverpayTxMessage)(everpayTxWithoutSig);
        const { sig, everHash } = await (0, sign_1.signRegisterAsync)(this._config, messageData, attachment);
        const everpayTx = {
            ...everpayTxWithoutSig,
            sig
        };
        const postEverpayTxResult = await (0, api_1.postTx)(this._apiHost, everpayTx);
        return {
            ...postEverpayTxResult,
            everpayTx,
            everHash
        };
    }
    async transfer(params) {
        const everpayTxWithoutSig = await this.getEverpayTxWithoutSig('transfer', params);
        return await this.sendEverpayTx(everpayTxWithoutSig);
    }
    async withdraw(params) {
        var _a, _b;
        if (((_a = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _a === void 0 ? void 0 : _a.value) == null) {
            await this.info();
        }
        const to = (_b = params.to) !== null && _b !== void 0 ? _b : this._config.account;
        const everpayTxWithoutSig = await this.getEverpayTxWithoutSig('withdraw', {
            ...params,
            to
        });
        return await this.sendEverpayTx(everpayTxWithoutSig);
    }
    async getBundleData(items, expiration, data) {
        var _a, _b, _c;
        if (((_a = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _a === void 0 ? void 0 : _a.value) == null) {
            await this.info();
        }
        return (0, util_1.genBundleData)({
            items,
            tokenList: (_c = (_b = _cachedInfo.everpay) === null || _b === void 0 ? void 0 : _b.value) === null || _c === void 0 ? void 0 : _c.tokenList,
            // 设置 60s 过期
            expiration: expiration !== null && expiration !== void 0 ? expiration : Math.round(Date.now() / 1000) + 60,
            data
        });
    }
    async signBundleData(bundleData) {
        const { items, expiration, salt, version, data } = bundleData;
        const { sig } = await (0, sign_1.signMessageAsync)(this._config, JSON.stringify({
            // 只签名这几个字段，并且顺序需要保持一致
            items, expiration, salt, version
        }));
        const sigs = bundleData.sigs != null ? bundleData.sigs : {};
        const acc = (0, util_1.isSmartAccount)(this._config.account) ? (0, util_1.genEverId)(this._config.account) : this._config.account;
        sigs[acc] = sig;
        return {
            items, expiration, salt, version, sigs, data
        };
    }
    async bundle(params) {
        const everpayTxWithoutSig = await this.getEverpayTxWithoutSig('bundle', params);
        return await this.sendEverpayTx(everpayTxWithoutSig);
    }
    async signAddTokenSet(newToken) {
        const addToken = {
            action: 'addToken',
            operator: this._config.account,
            salt: (0, uuid_1.v4)(),
            version: 'v1',
            expiration: Math.round(Date.now() / 1000) + 100,
            token: newToken,
            sig: ''
        };
        const { sig } = await (0, sign_1.signMessageAsync)(this._config, JSON.stringify({
            action: addToken.action,
            operator: addToken.operator,
            salt: addToken.salt,
            version: addToken.version,
            expiration: addToken.expiration,
            token: addToken.token
        }));
        addToken.sig = sig;
        return addToken;
    }
    async signAddTargetChainSet(tokenTag, targetChain) {
        const addTargetChain = {
            action: 'addTargetChain',
            operator: this._config.account,
            salt: (0, uuid_1.v4)(),
            version: 'v1',
            expiration: Math.round(Date.now() / 1000) + 100,
            tokenTag: tokenTag,
            targetChain: targetChain,
            sig: ''
        };
        const { sig } = await (0, sign_1.signMessageAsync)(this._config, JSON.stringify({
            action: addTargetChain.action,
            operator: addTargetChain.operator,
            salt: addTargetChain.salt,
            version: addTargetChain.version,
            expiration: addTargetChain.expiration,
            tokenTag: addTargetChain.tokenTag,
            targetChain: addTargetChain.targetChain
        }));
        addTargetChain.sig = sig;
        return addTargetChain;
    }
    async signTokenDisplaySet(tokenTag, display) {
        const tokenDisplay = {
            action: 'setTokenDisplay',
            operator: this._config.account,
            salt: (0, uuid_1.v4)(),
            version: 'v1',
            expiration: Math.round(Date.now() / 1000) + 100,
            tokenTag: tokenTag,
            display: display,
            sig: ''
        };
        const { sig } = await (0, sign_1.signMessageAsync)(this._config, JSON.stringify({
            action: tokenDisplay.action,
            operator: tokenDisplay.operator,
            salt: tokenDisplay.salt,
            version: tokenDisplay.version,
            expiration: tokenDisplay.expiration,
            tokenTag: tokenDisplay.tokenTag,
            display: tokenDisplay.display
        }));
        tokenDisplay.sig = sig;
        return tokenDisplay;
    }
    async signOwnershipSet(newOwner) {
        const ownership = {
            action: 'transferOwnership',
            operator: this._config.account,
            salt: (0, uuid_1.v4)(),
            version: 'v1',
            expiration: Math.round(Date.now() / 1000) + 100,
            newOwner: newOwner,
            sig: ''
        };
        const { sig } = await (0, sign_1.signMessageAsync)(this._config, JSON.stringify({
            action: ownership.action,
            operator: ownership.operator,
            salt: ownership.salt,
            version: ownership.version,
            expiration: ownership.expiration,
            newOwner: ownership.newOwner
        }));
        ownership.sig = sig;
        return ownership;
    }
    async setTx(setData) {
        const setParams = { amount: '0', data: setData, symbol: 'eth', to: this._config.account };
        const everpayTxWithoutSig = await this.getEverpayTxWithoutSig('set', setParams);
        return await this.sendEverpayTx(everpayTxWithoutSig);
    }
    async setAcc(data, accountData) {
        const params = {
            amount: '0',
            data,
            tag: 'ethereum-eth-0x0000000000000000000000000000000000000000',
            to: this._config.account
        };
        const everpayTxWithoutSig = await this.getEverpayTxWithoutSig('setAcc', params);
        const messageData = (0, sign_1.getEverpayTxMessage)(everpayTxWithoutSig);
        const { sig, everHash } = await (0, sign_1.signMessageAsync)(this._config, messageData, accountData);
        const everpayTx = {
            ...everpayTxWithoutSig,
            sig
        };
        const postEverpayTxResult = await (0, api_1.postTx)(this._apiHost, everpayTx);
        return {
            ...postEverpayTxResult,
            everpayTx,
            everHash
        };
    }
}
exports.default = Everpay;
