"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transferAsync = exports.getRedPackTxMessage = exports.signMessageAsync = exports.signRegisterAsync = exports.getEverpayTxMessage = exports.getDepositAddr = void 0;
const ethereum_1 = __importDefault(require("./ethereum"));
const arweave_1 = __importDefault(require("./arweave"));
const smartAccount_1 = __importDefault(require("./smartAccount"));
const types_1 = require("../types");
const check_1 = require("../utils/check");
const errors_1 = require("../utils/errors");
const hashPersonalMessage_1 = __importDefault(require("./hashPersonalMessage"));
const util_1 = require("../utils/util");
const popup_1 = require("./popup");
const getDepositAddr = (info, accountChainType, token) => {
    const symbol = token.symbol;
    if ((0, util_1.isArweaveAOSTestToken)(token)) {
        return info === null || info === void 0 ? void 0 : info.lockers.aostest;
    }
    if ((0, util_1.isPermaswapHaloTokenSymbol)(symbol)) {
        return info === null || info === void 0 ? void 0 : info.lockers.psntest;
    }
    if (accountChainType === types_1.ChainType.ethereum) {
        return info === null || info === void 0 ? void 0 : info.lockers.ethereum;
    }
    else if (accountChainType === types_1.ChainType.arweave) {
        // AR 大小写敏感
        return info === null || info === void 0 ? void 0 : info.lockers.arweave;
    }
    else if (accountChainType === types_1.ChainType.moon) {
        return info === null || info === void 0 ? void 0 : info.lockers.moon;
    }
    else if (accountChainType === types_1.ChainType.conflux) {
        return info === null || info === void 0 ? void 0 : info.lockers.conflux;
    }
    else if (accountChainType === types_1.ChainType.bsc) {
        return info === null || info === void 0 ? void 0 : info.lockers.bsc;
    }
    else if (accountChainType === types_1.ChainType.platon) {
        return info === null || info === void 0 ? void 0 : info.lockers.platon;
    }
    else if (accountChainType === types_1.ChainType.mapo) {
        return info === null || info === void 0 ? void 0 : info.lockers.mapo;
    }
    throw new Error(errors_1.ERRORS.INVALID_ACCOUNT_TYPE);
};
exports.getDepositAddr = getDepositAddr;
const getEverpayTxMessage = (everpayTxWithoutSig) => {
    const keys = [
        'tokenSymbol',
        'action',
        'from',
        'to',
        'amount',
        'fee',
        'feeRecipient',
        'nonce',
        'tokenID',
        'chainType',
        'chainID',
        'data',
        'version'
    ];
    return keys.map(key => `${key}:${everpayTxWithoutSig[key]}`).join('\n');
};
exports.getEverpayTxMessage = getEverpayTxMessage;
const signRegisterAsync = async (config, messageData, attachment) => {
    var _a;
    const from = config.account;
    const accountChainType = config.chainType;
    const personalMsgHashBuffer = (0, hashPersonalMessage_1.default)(Buffer.from(messageData));
    const personalMsgHex = `0x${personalMsgHashBuffer.toString('hex')}`;
    let sig = '';
    (0, check_1.checkSignConfig)(accountChainType, config);
    if ((_a = config.isSmartAccount) !== null && _a !== void 0 ? _a : false) {
        sig = await smartAccount_1.default.signRegisterAsync(Boolean(config.debug), Boolean(config.isSmartAccount), from, personalMsgHex, undefined, attachment);
    }
    else if ([
        types_1.ChainType.ethereum,
        types_1.ChainType.moon,
        types_1.ChainType.conflux,
        types_1.ChainType.bsc,
        types_1.ChainType.platon,
        types_1.ChainType.mapo
    ].includes(accountChainType)) {
        sig = await ethereum_1.default.signMessageAsync(Boolean(config.debug), config.ethConnectedSigner, from, messageData);
        sig = `${sig},,ECDSA`;
    }
    else if (accountChainType === types_1.ChainType.arweave) {
        sig = await arweave_1.default.signMessageAsync(Boolean(config.debug), config.arJWK, from, messageData);
        sig = `${sig},RSA`;
    }
    else {
        throw new Error(errors_1.ERRORS.INVALID_ACCOUNT_TYPE);
    }
    return { everHash: personalMsgHex, sig };
};
exports.signRegisterAsync = signRegisterAsync;
const signMessageAsync = async (config, messageData, accountData, directly) => {
    var _a, _b;
    const from = config.account;
    const accountChainType = config.chainType;
    const personalMsgHashBuffer = (0, hashPersonalMessage_1.default)(Buffer.from(messageData));
    const personalMsgHex = `0x${personalMsgHashBuffer.toString('hex')}`;
    let sig = '';
    (0, check_1.checkSignConfig)(accountChainType, config);
    if (!(0, util_1.isNodeJs)() && Boolean(config.isSmartAccount) && !window.location.host.includes('everpay.io')) {
        const url = `https://app${((_a = config.debug) !== null && _a !== void 0 ? _a : false) ? '-dev' : ''}.everpay.io/sign?account=${config.account}&host=${encodeURIComponent(window.location.host)}&directly=${directly ? '1' : '0'}`;
        // const url = `http://localhost:8080/sign?account=${config.account as string}&host=${encodeURIComponent(window.location.host)}&directly=${directly ? '1' : '0'}`
        const popup = await (0, popup_1.openPopup)(url);
        // 先接收到子窗口的 signPageLoaded 事件
        await (0, popup_1.runPopup)({
            popup,
            type: 'signPageLoaded',
            keepPopup: true
        });
        // 再向子窗口发送 messageData
        popup.postMessage(JSON.stringify({ data: encodeURIComponent(messageData), type: 'signPageMessageData' }), '*');
        sig = await (0, popup_1.runPopup)({
            popup,
            type: 'sign'
        });
    }
    else {
        if ((_b = config.isSmartAccount) !== null && _b !== void 0 ? _b : false) {
            sig = await smartAccount_1.default.signMessageAsync(Boolean(config.debug), Boolean(config.isSmartAccount), from, directly ? messageData : personalMsgHex, accountData);
        }
        else if ([
            types_1.ChainType.ethereum,
            types_1.ChainType.moon,
            types_1.ChainType.conflux,
            types_1.ChainType.bsc,
            types_1.ChainType.platon,
            types_1.ChainType.mapo
        ].includes(accountChainType)) {
            sig = await ethereum_1.default.signMessageAsync(Boolean(config.debug), config.ethConnectedSigner, from, messageData);
        }
        else if (accountChainType === types_1.ChainType.arweave) {
            sig = await arweave_1.default.signMessageAsync(Boolean(config.debug), config.arJWK, from, messageData);
        }
        else {
            throw new Error(errors_1.ERRORS.INVALID_ACCOUNT_TYPE);
        }
    }
    return { everHash: personalMsgHex, sig };
};
exports.signMessageAsync = signMessageAsync;
const getRedPackTxMessage = (redPackTxSig) => {
    const keys = [
        'redpacketUUID',
        'claimBy',
        'salt',
        'createdAt'
    ];
    return keys.map(key => `${key}:${redPackTxSig[key]}`).join('\n');
};
exports.getRedPackTxMessage = getRedPackTxMessage;
const transferAsync = async (config, info, params) => {
    (0, check_1.checkSignConfig)(config.chainType, config);
    const to = (0, exports.getDepositAddr)(info, config.chainType, params.token);
    const paramsMergedTo = { ...params, to };
    if ([
        types_1.ChainType.ethereum,
        types_1.ChainType.moon,
        types_1.ChainType.conflux,
        types_1.ChainType.bsc,
        types_1.ChainType.platon,
        types_1.ChainType.mapo
    ].includes(config.chainType)) {
        return await ethereum_1.default.transferAsync(config.ethConnectedSigner, config.chainType, paramsMergedTo);
    }
    else if (config.chainType === types_1.ChainType.arweave) {
        return await arweave_1.default.transferAsync(config.arJWK, config.chainType, paramsMergedTo);
    }
    throw new Error(errors_1.ERRORS.INVALID_ACCOUNT_TYPE);
};
exports.transferAsync = transferAsync;
