"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isMobile = exports.isMobileDevice = exports.isNodeJs = exports.genEverId = exports.hexToUint8Array = exports.uint8ArrayToHex = exports.getUserId = exports.isSmartAccount = exports.genBundleData = exports.genExpressData = exports.matchTokenTag = exports.genTokenTag = exports.getTokenBurnFeeByChainType = exports.getChainDecimalByChainType = exports.getTokenAddrByChainType = exports.getAccountChainType = exports.isPermaswapHaloTokenSymbol = exports.isArweaveAOSTestToken = exports.isArweaveL2PSTTokenSymbol = exports.isArweaveChainPSTMode = exports.isArweaveAddress = exports.isEthereumAddress = exports.getTokenByTag = exports.getTimestamp = exports.fromDecimalToUnit = exports.fromDecimalToUnitBN = exports.fromUnitToDecimal = exports.fromUnitToDecimalBN = exports.toBN = void 0;
const address_1 = require("@ethersproject/address");
const isString_1 = __importDefault(require("lodash/isString"));
const uuid_1 = require("uuid");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const errors_1 = require("./errors");
const types_1 = require("../types");
const config_1 = require("../config");
const sha256_1 = __importDefault(require("crypto-js/sha256"));
const enc_hex_1 = __importDefault(require("crypto-js/enc-hex"));
bignumber_js_1.default.config({
    EXPONENTIAL_AT: 1000,
});
const toBN = (x) => {
    if (isNaN(Number(x)))
        return new bignumber_js_1.default(0);
    if (x instanceof bignumber_js_1.default)
        return x;
    if (typeof x === 'string') {
        if (x.indexOf('0x') === 0 || x.indexOf('-0x') === 0) {
            return new bignumber_js_1.default((x).replace('0x', ''), 16);
        }
    }
    return new bignumber_js_1.default(x);
};
exports.toBN = toBN;
const fromUnitToDecimalBN = (x, decimals) => {
    return (0, exports.toBN)(x).times((0, exports.toBN)(10).pow(decimals));
};
exports.fromUnitToDecimalBN = fromUnitToDecimalBN;
const fromUnitToDecimal = (x, decimals) => {
    return (0, exports.fromUnitToDecimalBN)(x, decimals).toString();
};
exports.fromUnitToDecimal = fromUnitToDecimal;
const fromDecimalToUnitBN = (x, decimals) => {
    return (0, exports.toBN)(x).dividedBy((0, exports.toBN)(10).pow(decimals));
};
exports.fromDecimalToUnitBN = fromDecimalToUnitBN;
const fromDecimalToUnit = (x, decimals) => {
    return (0, exports.fromDecimalToUnitBN)(x, decimals).toString();
};
exports.fromDecimalToUnit = fromDecimalToUnit;
const getTimestamp = () => Math.round(Date.now() / 1000);
exports.getTimestamp = getTimestamp;
const getTokenByTag = (tag, tokenList) => {
    return tokenList === null || tokenList === void 0 ? void 0 : tokenList.find(t => (0, exports.matchTokenTag)((0, exports.genTokenTag)(t), tag));
};
exports.getTokenByTag = getTokenByTag;
exports.isEthereumAddress = address_1.isAddress;
const isArweaveAddress = (address) => {
    return (0, isString_1.default)(address) && address.length === 43 && address.search(/[a-z0-9A-Z_-]{43}/g) === 0;
};
exports.isArweaveAddress = isArweaveAddress;
const isArweaveChainPSTMode = (token) => {
    if (token == null)
        return false;
    return token.crossChainInfoList[types_1.ChainType.arweave] != null && token.symbol.toUpperCase() !== 'AR';
};
exports.isArweaveChainPSTMode = isArweaveChainPSTMode;
const isArweaveL2PSTTokenSymbol = (symbol) => {
    return ['STAMP', 'U'].includes(symbol === null || symbol === void 0 ? void 0 : symbol.toUpperCase());
};
exports.isArweaveL2PSTTokenSymbol = isArweaveL2PSTTokenSymbol;
const isArweaveAOSTestToken = (token) => {
    var _a;
    return token.chainType.includes(types_1.ChainType.aostest) && ['AOCRED', '0RBT', 'TRUNK', 'EXP', 'NEXP', 'EXP(ARIO)', 'BP', 'AR'].includes((_a = token === null || token === void 0 ? void 0 : token.symbol) === null || _a === void 0 ? void 0 : _a.toUpperCase());
};
exports.isArweaveAOSTestToken = isArweaveAOSTestToken;
const isPermaswapHaloTokenSymbol = (symbol) => {
    return (symbol === null || symbol === void 0 ? void 0 : symbol.toUpperCase()) === 'HALO';
};
exports.isPermaswapHaloTokenSymbol = isPermaswapHaloTokenSymbol;
const getAccountChainType = (from) => {
    if ((0, exports.isEthereumAddress)(from)) {
        return types_1.ChainType.ethereum;
    }
    if ((0, exports.isArweaveAddress)(from)) {
        return types_1.ChainType.arweave;
    }
    throw new Error(errors_1.ERRORS.INVALID_ACCOUNT_TYPE);
};
exports.getAccountChainType = getAccountChainType;
const getTokenAddrByChainType = (token, chainType) => {
    const crossChainInfo = token.crossChainInfoList[chainType];
    return crossChainInfo.targetTokenId;
};
exports.getTokenAddrByChainType = getTokenAddrByChainType;
const getChainDecimalByChainType = (token, chainType) => {
    const crossChainInfo = token.crossChainInfoList[chainType];
    return crossChainInfo.targetDecimals;
};
exports.getChainDecimalByChainType = getChainDecimalByChainType;
const getTokenBurnFeeByChainType = (token, feeItem, chainType) => {
    return feeItem.burnFeeMap[chainType];
};
exports.getTokenBurnFeeByChainType = getTokenBurnFeeByChainType;
const genTokenTag = (token) => {
    const { chainType, symbol, id } = token;
    const chainTypes = chainType.split(',');
    const tokenAddrs = id.split(',').map((addr, index) => {
        if ([
            types_1.ChainType.ethereum,
            types_1.ChainType.bsc,
            types_1.ChainType.conflux,
            types_1.ChainType.moon,
            types_1.ChainType.platon,
            'everpay'
        ].includes(chainTypes[index])) {
            return addr.toLowerCase();
        }
        return addr;
    });
    return `${chainType.toLowerCase()}-${symbol.toLowerCase()}-${tokenAddrs.join(',')}`;
};
exports.genTokenTag = genTokenTag;
const matchTokenTag = (tag1, tag2) => {
    return (tag1 === null || tag1 === void 0 ? void 0 : tag1.toLowerCase()) === (tag2 === null || tag2 === void 0 ? void 0 : tag2.toLowerCase());
};
exports.matchTokenTag = matchTokenTag;
const genExpressData = (params) => {
    const { chainType, to, fee } = params;
    return {
        appId: 'express',
        withdrawAction: 'pay',
        withdrawTo: to,
        withdrawChainType: chainType,
        withdrawFee: fee
    };
};
exports.genExpressData = genExpressData;
const genBundleData = (params) => {
    const items = params.items.map((item) => {
        const { tag, amount, from, to, data } = item;
        const token = (0, exports.getTokenByTag)(tag, params.tokenList);
        // 注意：顺序必须与后端保持一致，来让 JSON.stringify() 生成的字符串顺序与后端也一致
        return {
            tag: (0, exports.genTokenTag)(token),
            chainID: token.chainID,
            from,
            to,
            data: data != null ? data : '',
            amount: (0, exports.fromUnitToDecimal)(amount, token.decimals)
        };
    });
    const salt = (0, uuid_1.v4)();
    const version = config_1.bundleInternalTxVersion;
    return {
        // 注意：顺序必须与后端保持一致，来让 JSON.stringify() 生成的字符串顺序与后端也一致
        items,
        expiration: params.expiration,
        salt,
        version,
        data: params.data != null ? params.data : ''
    };
};
exports.genBundleData = genBundleData;
const isSmartAccount = (account) => {
    return /^\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$/.test(account);
};
exports.isSmartAccount = isSmartAccount;
const getUserId = (debug, everId, expiration) => {
    const everpayChainIdStr = debug ? '5' : '1';
    // hash the message
    const hash = (0, sha256_1.default)(`${everId.trim().toLowerCase()}${everpayChainIdStr}${expiration ? expiration : ''}`);
    const arrBuffer = (0, exports.hexToUint8Array)(hash.toString()).slice(0, 10);
    const b64encoded = window.btoa(String.fromCharCode.apply(null, arrBuffer));
    return b64encoded;
};
exports.getUserId = getUserId;
const checkSum = (idBytes) => {
    const hash = (0, sha256_1.default)(enc_hex_1.default.parse((0, exports.uint8ArrayToHex)(Uint8Array.from([...Buffer.from('eid'), ...idBytes]))));
    return (0, exports.hexToUint8Array)(hash.toString()).slice(0, 2);
};
const uint8ArrayToHex = (uint8Array) => {
    return [...uint8Array].map((b) => {
        return b.toString(16).padStart(2, '0');
    }).join('');
};
exports.uint8ArrayToHex = uint8ArrayToHex;
const hexToUint8Array = (hexString) => Uint8Array.from(hexString.match(/.{1,2}/g).map((byte) => parseInt(byte, 16)));
exports.hexToUint8Array = hexToUint8Array;
const genEverId = (email) => {
    const str = email.toLowerCase().trim();
    const hash = (0, sha256_1.default)(str);
    const idBytes = (0, exports.hexToUint8Array)(hash.toString());
    const sum = checkSum(idBytes);
    const concatArray = Uint8Array.from([...idBytes, ...sum]);
    return `eid${(0, exports.uint8ArrayToHex)(concatArray)}`;
};
exports.genEverId = genEverId;
const isNodeJs = () => typeof process !== 'undefined' &&
    process.versions != null &&
    process.versions.node != null;
exports.isNodeJs = isNodeJs;
const mobileRE = /(android|bb\d+|meego).+mobile|armv7l|avantgo|bada\/|ipad|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series[46]0|symbian|treo|up\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino/i;
const tabletRE = /android|ipad|playbook|silk/i;
const isMobileDevice = (opts) => {
    if (opts == null)
        opts = {};
    let ua = opts.ua;
    if (ua == null && typeof navigator !== 'undefined')
        ua = navigator.userAgent;
    if ((ua === null || ua === void 0 ? void 0 : ua.headers) != null && typeof ua.headers['user-agent'] === 'string') {
        ua = ua.headers['user-agent'];
    }
    if (typeof ua !== 'string')
        return false;
    let result = mobileRE.test(ua) || ((opts === null || opts === void 0 ? void 0 : opts.tablet) != null && tabletRE.test(ua));
    if (!result &&
        (opts === null || opts === void 0 ? void 0 : opts.tablet) != null &&
        (opts === null || opts === void 0 ? void 0 : opts.featureDetect) != null &&
        (navigator === null || navigator === void 0 ? void 0 : navigator.maxTouchPoints) > 1 &&
        ua.includes('Macintosh') &&
        ua.includes('Safari')) {
        result = true;
    }
    return result;
};
exports.isMobileDevice = isMobileDevice;
exports.isMobile = (0, exports.isMobileDevice)({
    tablet: true,
    featureDetect: true
});
