import { getEverpayTxMessage, signMessageAsync, signRegisterAsync, transferAsync, getRedPackTxMessage } from './lib/sign';
import { getEverpayBalance, getEverpayBalances, getEverpayInfo, getExpressInfo, getExplorerTransactions, getMintdEverpayTransactionByChainTxHash, postTx, getFees, getFee, getEmailRegisterData, getAccountData, verifySig, getExplorerTransaction } from './api';
import { everpayTxVersion, getExpressHost, getEverpayHost, getExplorerHost } from './config';
import { getTimestamp, toBN, getAccountChainType, fromDecimalToUnit, genTokenTag, matchTokenTag, genExpressData, fromUnitToDecimalBN, genBundleData, getTokenBurnFeeByChainType, getChainDecimalByChainType, isArweaveChainPSTMode, getTokenByTag, isArweaveL2PSTTokenSymbol, isSmartAccount, genEverId, isArweaveAddress, uint8ArrayToHex, isEthereumAddress, isArweaveAOSTestToken } from './utils/util';
import { checkParams } from './utils/check';
import sha256 from 'crypto-js/sha256';
import { EverpayBase, EverpayAction, ChainType } from './types';
import { ERRORS } from './utils/errors';
import { utils } from 'ethers';
import { v4 as uuidv4 } from 'uuid';
import { openPopup, runPopup } from './lib/popup';
import { EVERPAY_JS_VERSION } from './constants';
import hashPersonalMessage from './lib/hashPersonalMessage';
import isString from 'lodash/isString';
export * from './types';
const _cachedInfo = {};
const cacheHelper = async (key, host) => {
    var _a, _b, _c;
    const timestamp = getTimestamp();
    // cache info 3 mins
    if (((_a = _cachedInfo[key]) === null || _a === void 0 ? void 0 : _a.value) != null &&
        _cachedInfo[key].timestamp > timestamp - 3 * 60) {
        return (_b = _cachedInfo[key]) === null || _b === void 0 ? void 0 : _b.value;
    }
    if (key === 'everpay') {
        const value = await getEverpayInfo(host);
        _cachedInfo[key] = { value, timestamp };
    }
    else if (key === 'express') {
        const value = await getExpressInfo(host);
        _cachedInfo[key] = { value, timestamp };
    }
    return (_c = _cachedInfo[key]) === null || _c === void 0 ? void 0 : _c.value;
};
class Everpay extends EverpayBase {
    constructor(config) {
        var _a, _b;
        super();
        this.getAccountChainType = getAccountChainType;
        this._config = {
            ...config,
            account: (_a = config === null || config === void 0 ? void 0 : config.account) !== null && _a !== void 0 ? _a : '',
            chainType: (_b = config === null || config === void 0 ? void 0 : config.chainType) !== null && _b !== void 0 ? _b : ChainType.ethereum
        };
        this._apiHost = getEverpayHost(config === null || config === void 0 ? void 0 : config.debug);
        this._expressHost = getExpressHost(config === null || config === void 0 ? void 0 : config.debug);
        this._explorerHost = getExplorerHost(config === null || config === void 0 ? void 0 : config.debug);
    }
    async info() {
        const result = await cacheHelper('everpay', this._apiHost);
        return result;
    }
    async expressInfo() {
        const result = await cacheHelper('express', this._expressHost);
        return result;
    }
    async getEmailRegisterData() {
        const result = await getEmailRegisterData(this._apiHost, this._config.account);
        return result;
    }
    async getAccountData() {
        const acc = isSmartAccount(this._config.account) ? genEverId(this._config.account) : this._config.account;
        const result = await getAccountData(this._apiHost, acc);
        return result;
    }
    async balance(params) {
        var _a;
        await this.info();
        const { tag, account } = params;
        const accTemp = account !== null && account !== void 0 ? account : this._config.account;
        const acc = isSmartAccount(accTemp) ? genEverId(accTemp) : accTemp;
        const token = getTokenByTag(tag, (_a = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _a === void 0 ? void 0 : _a.value.tokenList);
        checkParams({ account: acc, tag, token });
        const mergedParams = {
            tokenTag: genTokenTag(token),
            account: acc
        };
        const everpayBalance = await getEverpayBalance(this._apiHost, mergedParams);
        return fromDecimalToUnit(everpayBalance.balance.amount, everpayBalance.balance.decimals);
    }
    async balances(params) {
        const info = await this.info();
        params = (params !== null && params !== void 0 ? params : {});
        const { account } = params;
        const accTemp = account !== null && account !== void 0 ? account : this._config.account;
        const acc = isSmartAccount(accTemp) ? genEverId(accTemp) : accTemp;
        checkParams({ account: acc });
        const mergedParams = {
            account: acc
        };
        const everpayBalances = await getEverpayBalances(this._apiHost, mergedParams);
        const argTag = !this._config.debug ? 'bsc-arg-0xb5eadfdbdb40257d1d24a1432faa2503a867c270' : 'bsc-arg-0x7846cf6e181bb5c909d6010d15af5fffd3b61229';
        const fraTag = !this._config.debug ? 'bsc-fra-0xeb042ffdabc535de2716c6b51a965f124050d4e1' : 'bsc-fra-0xa98242557818f0135b2381893caec3d4a64f88e5';
        const deleteTag = [
            argTag,
            fraTag
        ];
        const balances = everpayBalances.balances.filter((t) => t.tag !== deleteTag[0] && t.tag !== deleteTag[1]).map(item => {
            const tag = item.tag;
            const token = info.tokenList.find(token => token.tag === tag);
            return {
                chainType: token === null || token === void 0 ? void 0 : token.chainType,
                symbol: token === null || token === void 0 ? void 0 : token.symbol.toUpperCase(),
                tag: token === null || token === void 0 ? void 0 : token.tag,
                address: token === null || token === void 0 ? void 0 : token.id,
                balance: fromDecimalToUnit(item.amount, item.decimals)
            };
        });
        return balances;
    }
    async getMergedTxsParams(params) {
        var _a;
        const { page, tag, action, withoutAction, cursor } = params;
        const mergedParams = {};
        if (page !== undefined) {
            mergedParams.page = page;
        }
        if (cursor !== undefined) {
            mergedParams.cursor = cursor;
        }
        if (tag !== undefined) {
            await this.info();
            const token = getTokenByTag(tag, (_a = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _a === void 0 ? void 0 : _a.value.tokenList);
            checkParams({ token });
            mergedParams.tokenTag = token.tag;
        }
        if (action !== undefined) {
            checkParams({ action });
            mergedParams.action = action;
        }
        if (withoutAction !== undefined) {
            mergedParams.withoutAction = withoutAction;
        }
        return mergedParams;
    }
    async txs(params) {
        const mergedParams = await this.getMergedTxsParams(params);
        return await getExplorerTransactions(this._explorerHost, mergedParams);
    }
    async txsByAccount(params) {
        var _a;
        const accTemp = (_a = params.account) !== null && _a !== void 0 ? _a : this._config.account;
        const acc = isSmartAccount(accTemp) ? genEverId(accTemp) : accTemp;
        checkParams({ account: acc });
        const mergedParams = await this.getMergedTxsParams(params);
        mergedParams.account = acc;
        return await getExplorerTransactions(this._explorerHost, mergedParams);
    }
    async txByHash(everHash) {
        checkParams({ everHash });
        return await getExplorerTransaction(this._explorerHost, everHash);
    }
    async mintedTxByChainTxHash(chainTxHash) {
        checkParams({ chainTxHash });
        return await getMintdEverpayTransactionByChainTxHash(this._apiHost, chainTxHash);
    }
    async fees() {
        return await getFees(this._apiHost);
    }
    async fee(tag) {
        var _a;
        await this.info();
        const token = getTokenByTag(tag, (_a = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _a === void 0 ? void 0 : _a.value.tokenList);
        checkParams({ tag, token });
        return await getFee(this._apiHost, genTokenTag(token));
    }
    async smartAccountAuth(logo, email, emailEditable) {
        const debug = Boolean(this._config.debug);
        email = isSmartAccount(email) ? email : '';
        // eslint-disable-next-line no-unneeded-ternary
        emailEditable = emailEditable === false ? false : true;
        const url = `https://app${debug ? '-dev' : ''}.everpay.io/auth?host=${encodeURIComponent(window.location.host)}&logo=${encodeURIComponent(logo)}&version=${EVERPAY_JS_VERSION}&email=${email}&editable=${emailEditable ? 1 : 0}`;
        // const url = `http://localhost:8080/auth?host=${encodeURIComponent(window.location.host)}&logo=${encodeURIComponent(logo)}&version=${EVERPAY_JS_VERSION}&email=${email}&editable=${emailEditable ? 1 : 0}`
        const popup = await openPopup(url);
        return await runPopup({
            popup,
            type: 'auth'
        });
    }
    genEverId(account) {
        const acc = account !== null && account !== void 0 ? account : this._config.account;
        return isSmartAccount(acc) ? genEverId(acc) : acc;
    }
    hashMessage(message) {
        const personalMsgHashBuffer = hashPersonalMessage(Buffer.from(message));
        const personalMsgHex = `0x${personalMsgHashBuffer.toString('hex')}`;
        return personalMsgHex;
    }
    async signMessage(message, smartAccountDirectly) {
        if (!isString(message)) {
            throw new Error(ERRORS.MESSAGE_INCORRECT);
        }
        if (isSmartAccount(this._config.account)) {
            if (smartAccountDirectly != null) {
                if (message.length >= 96) {
                    const { sig } = await signMessageAsync(this._config, message, undefined, true);
                    return { message, sig };
                }
                else {
                    throw new Error(ERRORS.MESSAGE_INCORRECT);
                }
            }
            else {
                const personalMsgHashBuffer = hashPersonalMessage(Buffer.from(message));
                const messageHash = `0x${personalMsgHashBuffer.toString('hex')}`;
                const { sig } = await signMessageAsync(this._config, message);
                return { message: messageHash, sig };
            }
        }
        else {
            const { sig } = await signMessageAsync(this._config, message);
            return { message, sig };
        }
    }
    async verifyMessage(params) {
        const { account, message, type, sig } = params;
        const everId = this.genEverId(account);
        let messageHash = '';
        if (isEthereumAddress(account)) {
            const personalMsgHashBuffer = hashPersonalMessage(Buffer.from(message));
            messageHash = `0x${personalMsgHashBuffer.toString('hex')}`;
        }
        else if (isArweaveAddress(account)) {
            messageHash = `0x${sha256(message).toString()}`;
        }
        else if (isSmartAccount(account)) {
            messageHash = `0x${uint8ArrayToHex(Buffer.from(message))}`;
        }
        else {
            throw new Error(ERRORS.ACCOUNT_INVALID);
        }
        const [splitSig, splitPublic] = sig.split(',');
        return await verifySig(this._apiHost, {
            account: everId,
            message: messageHash,
            type,
            sig: splitSig,
            public: splitPublic
        });
    }
    async deposit(params) {
        var _a, _b;
        await this.info();
        const { amount, tag } = params;
        const from = this._config.account;
        const token = getTokenByTag(tag, (_a = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _a === void 0 ? void 0 : _a.value.tokenList);
        const chainType = this._config.chainType;
        checkParams({ account: from, tag, token, amount });
        // arweave 上的 PST 充值必须是整数
        if (isArweaveChainPSTMode(token) && chainType === ChainType.arweave && !isArweaveL2PSTTokenSymbol(token.symbol) && parseInt(amount) !== +amount) {
            throw new Error(ERRORS.DEPOSIT_ARWEAVE_PST_MUST_BE_INTEGER);
        }
        const chainDecimal = getChainDecimalByChainType(token, isArweaveAOSTestToken(token) ? ChainType.aostest : chainType);
        const value = utils.parseUnits(toBN(amount).toString(), chainDecimal);
        return await transferAsync(this._config, (_b = _cachedInfo.everpay) === null || _b === void 0 ? void 0 : _b.value, {
            symbol: token.symbol,
            token,
            from: from !== null && from !== void 0 ? from : '',
            value
        });
    }
    // amount 为实际收款数量
    async getEverpayTxWithoutSig(type, params) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q;
        if (((_a = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _a === void 0 ? void 0 : _a.value) == null) {
            await this.info();
        }
        const { tag, amount, fee, quickMode } = params;
        const token = getTokenByTag(tag, (_b = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _b === void 0 ? void 0 : _b.value.tokenList);
        const from = this._config.account;
        let data = params.data;
        let to = params === null || params === void 0 ? void 0 : params.to;
        let decimalFeeBN = toBN(0);
        let decimalOperateAmountBN = toBN(0);
        let action = EverpayAction.transfer;
        checkParams({ account: from, tag, token, to });
        if (type === 'transfer') {
            checkParams({ amount });
            action = EverpayAction.transfer;
            decimalOperateAmountBN = fromUnitToDecimalBN(amount, (_c = token === null || token === void 0 ? void 0 : token.decimals) !== null && _c !== void 0 ? _c : 0);
        }
        else if (type === 'bundle') {
            action = EverpayAction.bundle;
            decimalOperateAmountBN = fromUnitToDecimalBN(amount, (_d = token === null || token === void 0 ? void 0 : token.decimals) !== null && _d !== void 0 ? _d : 0);
            // eslint-disable-next-line @typescript-eslint/strict-boolean-expressions
        }
        else if (type === 'set') {
            action = EverpayAction.set;
            decimalOperateAmountBN = fromUnitToDecimalBN(amount, (_e = token === null || token === void 0 ? void 0 : token.decimals) !== null && _e !== void 0 ? _e : 0);
        }
        else if (type === 'setAcc') {
            action = EverpayAction.setAcc;
            decimalOperateAmountBN = fromUnitToDecimalBN(amount, (_f = token === null || token === void 0 ? void 0 : token.decimals) !== null && _f !== void 0 ? _f : 0);
        }
        else if (type === 'withdraw') {
            checkParams({ amount });
            const chainType = params.chainType;
            // PST 提现到 arweave 网络必须是整数
            if (isArweaveChainPSTMode(token) && chainType === ChainType.arweave && !isArweaveL2PSTTokenSymbol(token === null || token === void 0 ? void 0 : token.symbol) && parseInt(amount) !== +amount) {
                throw new Error(ERRORS.PST_WITHDARW_TO_ARWEAVE_MUST_BE_INTEGER);
            }
            const balance = await this.balance({ tag });
            const decimalBalanceBN = fromUnitToDecimalBN(balance, (_g = token === null || token === void 0 ? void 0 : token.decimals) !== null && _g !== void 0 ? _g : 0);
            // 快速提现
            if (quickMode === true) {
                action = EverpayAction.transfer;
                const expressInfo = await this.expressInfo();
                const tokenTag = genTokenTag(token);
                const foundExpressTokenData = expressInfo.tokens.find(t => matchTokenTag(tokenTag, t.tokenTag));
                if (foundExpressTokenData == null) {
                    throw new Error(ERRORS.WITHDRAW_TOKEN_NOT_SUPPORT_QUICK_MODE);
                }
                const quickWithdrawLimitBN = fromUnitToDecimalBN(foundExpressTokenData.walletBalance, (_h = token === null || token === void 0 ? void 0 : token.decimals) !== null && _h !== void 0 ? _h : 0);
                // 快速提现的手续费，只放入 data 字段中
                const quickWithdrawFeeBN = fee !== undefined
                    ? fromUnitToDecimalBN(fee, (_j = token === null || token === void 0 ? void 0 : token.decimals) !== null && _j !== void 0 ? _j : 0)
                    : toBN(foundExpressTokenData.withdrawFee);
                // 快速提现的 amount 为全部数量
                decimalOperateAmountBN = fromUnitToDecimalBN(amount, (_k = token === null || token === void 0 ? void 0 : token.decimals) !== null && _k !== void 0 ? _k : 0).plus(quickWithdrawFeeBN);
                if (decimalOperateAmountBN.gt(decimalBalanceBN)) {
                    throw new Error(ERRORS.WITHDRAW_AMOUNT_LESS_THAN_FEE);
                }
                if (decimalOperateAmountBN.gt(quickWithdrawLimitBN)) {
                    throw new Error(ERRORS.INSUFFICIENT_QUICK_WITHDRAWAL_AMOUNT);
                }
                const expressData = genExpressData({
                    chainType, to, fee: quickWithdrawFeeBN.toString()
                });
                data = data !== undefined ? { ...data, ...expressData } : { ...expressData };
                // to 需要更改为快速提现收款账户
                to = expressInfo.address;
                // 普通提现
            }
            else {
                action = EverpayAction.withdraw;
                if (fee !== undefined) {
                    decimalFeeBN = fromUnitToDecimalBN(fee, (_l = token === null || token === void 0 ? void 0 : token.decimals) !== null && _l !== void 0 ? _l : 0);
                }
                else {
                    const feeItem = await getFee(this._apiHost, genTokenTag(token));
                    decimalFeeBN = toBN((_m = getTokenBurnFeeByChainType(token, feeItem, chainType)) !== null && _m !== void 0 ? _m : '0');
                }
                const targetChainType = chainType;
                data = data !== undefined ? { ...data, targetChainType } : { targetChainType };
                decimalOperateAmountBN = fromUnitToDecimalBN(amount, (_o = token === null || token === void 0 ? void 0 : token.decimals) !== null && _o !== void 0 ? _o : 0);
                if (decimalOperateAmountBN.plus(decimalFeeBN).gt(decimalBalanceBN)) {
                    throw new Error(ERRORS.WITHDRAW_AMOUNT_LESS_THAN_FEE);
                }
            }
        }
        const everpayTxWithoutSig = {
            tokenSymbol: token === null || token === void 0 ? void 0 : token.symbol,
            action,
            from: isSmartAccount(from) ? genEverId(from) : from,
            to: isSmartAccount(to) ? genEverId(to) : to,
            amount: decimalOperateAmountBN.toString(),
            fee: decimalFeeBN.toString(),
            feeRecipient: (_q = (_p = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _p === void 0 ? void 0 : _p.value.feeRecipient) !== null && _q !== void 0 ? _q : '',
            nonce: Date.now().toString(),
            tokenID: token === null || token === void 0 ? void 0 : token.id,
            chainType: token === null || token === void 0 ? void 0 : token.chainType,
            chainID: token === null || token === void 0 ? void 0 : token.chainID,
            data: data !== undefined ? JSON.stringify(data) : '',
            version: everpayTxVersion
        };
        return everpayTxWithoutSig;
    }
    getEverpayTxMessage(everpayTxWithoutSig) {
        return getEverpayTxMessage(everpayTxWithoutSig);
    }
    async signedEverpayTx(everpayTxWithoutSig) {
        const messageData = getEverpayTxMessage(everpayTxWithoutSig);
        const { sig, everHash } = await signMessageAsync(this._config, messageData);
        const everpayTx = {
            ...everpayTxWithoutSig,
            sig
        };
        return { everpayTx, everHash };
    }
    // sig redpacket
    async signedRedPackTx(redPackTxSig) {
        const messageData = getRedPackTxMessage(redPackTxSig);
        const { sig, everHash } = await signMessageAsync(this._config, messageData);
        const redpackTx = {
            ...redPackTxSig,
            signature: sig
        };
        return { redpackTx, everHash };
    }
    async sendEverpayTx(everpayTxWithoutSig) {
        const { everpayTx, everHash } = await this.signedEverpayTx(everpayTxWithoutSig);
        const postEverpayTxResult = await postTx(this._apiHost, everpayTx);
        return {
            ...postEverpayTxResult,
            everpayTx,
            everHash
        };
    }
    async register(params, attachment) {
        var _a, _b, _c, _d;
        if (((_a = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _a === void 0 ? void 0 : _a.value) == null) {
            await this.info();
        }
        const everpayInfo = (_b = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _b === void 0 ? void 0 : _b.value;
        const token = everpayInfo.tokenList.find((t) => {
            return t.symbol.toUpperCase() === 'ETH';
        });
        const everpayTxWithoutSig = {
            tokenSymbol: token === null || token === void 0 ? void 0 : token.symbol,
            action: 'register',
            from: isSmartAccount(this._config.account) ? genEverId(this._config.account) : this._config.account,
            to: isSmartAccount(this._config.account) ? genEverId(this._config.account) : this._config.account,
            amount: '0',
            fee: '0',
            feeRecipient: (_d = (_c = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _c === void 0 ? void 0 : _c.value.feeRecipient) !== null && _d !== void 0 ? _d : '',
            nonce: Date.now().toString(),
            tokenID: token === null || token === void 0 ? void 0 : token.id,
            chainType: token === null || token === void 0 ? void 0 : token.chainType,
            chainID: token === null || token === void 0 ? void 0 : token.chainID,
            data: params !== undefined ? JSON.stringify({ mailVerify: params }) : '',
            version: everpayTxVersion
        };
        const messageData = getEverpayTxMessage(everpayTxWithoutSig);
        const { sig, everHash } = await signRegisterAsync(this._config, messageData, attachment);
        const everpayTx = {
            ...everpayTxWithoutSig,
            sig
        };
        const postEverpayTxResult = await postTx(this._apiHost, everpayTx);
        return {
            ...postEverpayTxResult,
            everpayTx,
            everHash
        };
    }
    async transfer(params) {
        const everpayTxWithoutSig = await this.getEverpayTxWithoutSig('transfer', params);
        return await this.sendEverpayTx(everpayTxWithoutSig);
    }
    async withdraw(params) {
        var _a, _b;
        if (((_a = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _a === void 0 ? void 0 : _a.value) == null) {
            await this.info();
        }
        const to = (_b = params.to) !== null && _b !== void 0 ? _b : this._config.account;
        const everpayTxWithoutSig = await this.getEverpayTxWithoutSig('withdraw', {
            ...params,
            to
        });
        return await this.sendEverpayTx(everpayTxWithoutSig);
    }
    async getBundleData(items, expiration, data) {
        var _a, _b, _c;
        if (((_a = _cachedInfo === null || _cachedInfo === void 0 ? void 0 : _cachedInfo.everpay) === null || _a === void 0 ? void 0 : _a.value) == null) {
            await this.info();
        }
        return genBundleData({
            items,
            tokenList: (_c = (_b = _cachedInfo.everpay) === null || _b === void 0 ? void 0 : _b.value) === null || _c === void 0 ? void 0 : _c.tokenList,
            // 设置 60s 过期
            expiration: expiration !== null && expiration !== void 0 ? expiration : Math.round(Date.now() / 1000) + 60,
            data
        });
    }
    async signBundleData(bundleData) {
        const { items, expiration, salt, version, data } = bundleData;
        const { sig } = await signMessageAsync(this._config, JSON.stringify({
            // 只签名这几个字段，并且顺序需要保持一致
            items, expiration, salt, version
        }));
        const sigs = bundleData.sigs != null ? bundleData.sigs : {};
        const acc = isSmartAccount(this._config.account) ? genEverId(this._config.account) : this._config.account;
        sigs[acc] = sig;
        return {
            items, expiration, salt, version, sigs, data
        };
    }
    async bundle(params) {
        const everpayTxWithoutSig = await this.getEverpayTxWithoutSig('bundle', params);
        return await this.sendEverpayTx(everpayTxWithoutSig);
    }
    async signAddTokenSet(newToken) {
        const addToken = {
            action: 'addToken',
            operator: this._config.account,
            salt: uuidv4(),
            version: 'v1',
            expiration: Math.round(Date.now() / 1000) + 100,
            token: newToken,
            sig: ''
        };
        const { sig } = await signMessageAsync(this._config, JSON.stringify({
            action: addToken.action,
            operator: addToken.operator,
            salt: addToken.salt,
            version: addToken.version,
            expiration: addToken.expiration,
            token: addToken.token
        }));
        addToken.sig = sig;
        return addToken;
    }
    async signAddTargetChainSet(tokenTag, targetChain) {
        const addTargetChain = {
            action: 'addTargetChain',
            operator: this._config.account,
            salt: uuidv4(),
            version: 'v1',
            expiration: Math.round(Date.now() / 1000) + 100,
            tokenTag: tokenTag,
            targetChain: targetChain,
            sig: ''
        };
        const { sig } = await signMessageAsync(this._config, JSON.stringify({
            action: addTargetChain.action,
            operator: addTargetChain.operator,
            salt: addTargetChain.salt,
            version: addTargetChain.version,
            expiration: addTargetChain.expiration,
            tokenTag: addTargetChain.tokenTag,
            targetChain: addTargetChain.targetChain
        }));
        addTargetChain.sig = sig;
        return addTargetChain;
    }
    async signTokenDisplaySet(tokenTag, display) {
        const tokenDisplay = {
            action: 'setTokenDisplay',
            operator: this._config.account,
            salt: uuidv4(),
            version: 'v1',
            expiration: Math.round(Date.now() / 1000) + 100,
            tokenTag: tokenTag,
            display: display,
            sig: ''
        };
        const { sig } = await signMessageAsync(this._config, JSON.stringify({
            action: tokenDisplay.action,
            operator: tokenDisplay.operator,
            salt: tokenDisplay.salt,
            version: tokenDisplay.version,
            expiration: tokenDisplay.expiration,
            tokenTag: tokenDisplay.tokenTag,
            display: tokenDisplay.display
        }));
        tokenDisplay.sig = sig;
        return tokenDisplay;
    }
    async signOwnershipSet(newOwner) {
        const ownership = {
            action: 'transferOwnership',
            operator: this._config.account,
            salt: uuidv4(),
            version: 'v1',
            expiration: Math.round(Date.now() / 1000) + 100,
            newOwner: newOwner,
            sig: ''
        };
        const { sig } = await signMessageAsync(this._config, JSON.stringify({
            action: ownership.action,
            operator: ownership.operator,
            salt: ownership.salt,
            version: ownership.version,
            expiration: ownership.expiration,
            newOwner: ownership.newOwner
        }));
        ownership.sig = sig;
        return ownership;
    }
    async setTx(setData) {
        const setParams = { amount: '0', data: setData, symbol: 'eth', to: this._config.account };
        const everpayTxWithoutSig = await this.getEverpayTxWithoutSig('set', setParams);
        return await this.sendEverpayTx(everpayTxWithoutSig);
    }
    async setAcc(data, accountData) {
        const params = {
            amount: '0',
            data,
            tag: 'ethereum-eth-0x0000000000000000000000000000000000000000',
            to: this._config.account
        };
        const everpayTxWithoutSig = await this.getEverpayTxWithoutSig('setAcc', params);
        const messageData = getEverpayTxMessage(everpayTxWithoutSig);
        const { sig, everHash } = await signMessageAsync(this._config, messageData, accountData);
        const everpayTx = {
            ...everpayTxWithoutSig,
            sig
        };
        const postEverpayTxResult = await postTx(this._apiHost, everpayTx);
        return {
            ...postEverpayTxResult,
            everpayTx,
            everHash
        };
    }
}
export default Everpay;
